@@ -2,20 +2,25 @@
 # t/testm.sh File::Temp
 # => $^X -Mblib blib/script/perlcc -S -e 'use File::Temp; print "ok"' -o file_temp
 #
-# How to installed skip modules:
-#  grep ^skip log.modules-bla|cut -c6-| xargs perlbla -S cpan
-#  perl5.11.4-nt -S cpan `grep -v '#' t/mymodules`
+# How to installed skip modules: 
+# t/testm.sh -s runs:
+#   grep ^skip log.modules-$ver|cut -c6-| xargs perl$ver -S cpan
+# perl$ver -S cpan `grep -v '#' t/mymodules`
 #
-# -t run CPAN::Shell->testcc($module)
+# -t runs CPAN::Shell->testcc($module)
 
 function help {
   echo "t/testm.sh [OPTIONS] [module|modules-file]..."
+  echo " -q                 quiet"
   echo " -k                 keep temp. files on PASS"
   echo " -D<arg>            add debugging flags"
+  echo " -f<arg>            add optimisation flags"
   echo " -l                 log"
   echo " -o                 orig. no -Mblib, use installed modules (5.6, 5.8)"
   echo " -t                 run the module tests also, not only use Module (experimental)"
+  echo " -F                 run failing modules only"
   echo " -s                 install skipped (missing) modules"
+  echo " -T                 time compilation and run"
   echo " -h                 help"
 }
 
@@ -43,23 +48,33 @@ function fail {
     echo
 }
 
-while getopts "hokltTsD:" opt
+while getopts "hokltTsFD:O:f:q" opt
 do
   if [ "$opt" = "o" ]; then Mblib=" "; init; fi
+  if [ "$opt" = "q" ]; then QUIET=1; fi
   if [ "$opt" = "k" ]; then KEEP="-S"; fi
-  if [ "$opt" = "D" ]; then KEEP="-D${OPTARG}"; fi
+  if [ "$opt" = "D" ]; then PERLCC_OPTS="$PERLCC_OPTS -Wb=-D${OPTARG}"; COPTS="$COPTS,-D${OPTARG}"; fi
+  if [ "$opt" = "O" ]; then PERLCC_OPTS="$PERLCC_OPTS -Wb=-O${OPTARG}"; COPTS="$COPTS,-O${OPTARG}"; fi
+  if [ "$opt" = "f" ]; then PERLCC_OPTS="$PERLCC_OPTS -Wb=-f${OPTARG}"; COPTS="$COPTS,-f${OPTARG}"; fi
   if [ "$opt" = "l" ]; then TEST="-log"; fi
   if [ "$opt" = "t" ]; then TEST="-t"; fi
-  if [ "$opt" = "T" ]; then PERLCC_OPTS="--time"; PERLCC_TIMEOUT=120; fi
+  if [ "$opt" = "T" ]; then PERLCC_OPTS="$PERLCC_OPTS --time"; PERLCC_TIMEOUT=120; fi
   if [ "$opt" = "s" ]; then 
       v=$($PERL -It -Mmodules -e'print perlversion')
       if [ -f log.modules-$v ]; then # and not older than a few days
-          grep ^skip log.modules-$v | cut -c6- | xargs $PERL -S cpan
+          grep ^skip log.modules-$v | perl -anle 'print $F[1]' | xargs $PERL -S cpan
       else
           $PERL -S cpan $($PERL $Mblib -It -Mmodules -e'$,=" "; print skip_modules')
       fi
       exit
   fi
+  if [ "$opt" = "F" ]; then 
+      v=$($PERL -It -Mmodules -e'print perlversion')
+      if [ -f log.modules-$v ]; then # and not older than a few days
+          grep ^fail log.modules-$v | perl -anle 'print $F[1]' | xargs $PERL t/testm.sh
+      fi
+      exit
+  fi
   if [ "$opt" = "h" ]; then help; exit; fi
 done
 
@@ -79,11 +94,12 @@ if [ -n "$1" ]; then
 	$PERL $Mblib t/modules.t $TEST "$1"
     else
 	while [ -n "$1" ]; do
-	    # single module
-	    name="$(perl -e'$ARGV[0]=~s{::}{_}g; print lc($ARGV[0])' $1)"
-	    if [ "${KEEP:0:2}" = "-D" ]; then
-	      echo $PERL $Mblib -MO=C,$KEEP,-o$name.c -e "\"use $1; print 'ok'\""
-	      $PERL $Mblib -MO=C,$KEEP,-o$name.c -e "use $1; print 'ok'"
+	    # single module. update,setup,install are UAC terms
+	    name="$(perl -e'$a=shift;$a=~s{::}{_}g;$a=~s{(install|setup|update)}{substr($1,0,4)}ie;print lc($a)' $1)"
+	    if [ "${COPTS/,-D/}" != "$COPTS" ]; then
+              COPTS="${COPTS:1}"
+	      echo $PERL $Mblib -MO=C,$COPTS,-o$name.c -e "\"use $1; print 'ok'\""
+	      $PERL $Mblib -MO=C,$COPTS,-o$name.c -e "use $1; print 'ok'"
 	      if [ -f $name.c ]; then
 		echo $PERL $Mblib script/cc_harness -d -g3 -o$name $name.c
 		$PERL $Mblib script/cc_harness -d -g3 -o$name $name.c
@@ -93,10 +109,11 @@ if [ -n "$1" ]; then
 		fi
 	      fi
 	    else
+	      [ -z "$QUIET" ] && PERLCC_OPTS="$PERLCC_OPTS -v 4"
 	      echo $PERL $Mblib blib/script/perlcc $PERLCC_OPTS -r $KEEP -e "\"use $1; print 'ok'\"" -o $name
 	      $PERL $Mblib blib/script/perlcc $PERLCC_OPTS -r $KEEP -e "use $1; print 'ok'" -o $name
+              test -f a.out.c && mv a.out.c $name.c
             fi
-	    test -f a.out.c && mv a.out.c $name.c
 	    [ -n "$TEST" ] && $PERL $Mblib -It -MCPAN -Mmodules -e"CPAN::Shell->testcc(q($1))"
 	    shift
 	done
